@@ -6,9 +6,6 @@ import random
 import curses
 import cursor
 import requests
-import threading
-import ctypes
-import configparser
 import json
 import zipfile
 import re
@@ -35,7 +32,7 @@ magenta=_magenta
 cyan=_cyan
 white=_white
 grey=_grey
-_version='N0Lib:0.0.4'
+_version='N0Lib:0.0.5'
 __N0Version__=''
 __SETVERSION__=False
 __DEBUG__=False 
@@ -43,9 +40,10 @@ __GRAPHIC__=False
 __dirChar__='/'  
 __WORKPATH__='./'
 __SHELLPATH__='./'
-__FLITERMODE__='half'
-__COLORFLITER__='$n0p'
-__COLORFLITERS__={'c3h7n3o9':[161,209,215,84,88,40,34,58],
+__FILTERMODE__='half'
+__COLORFILTER__='$n0p'
+__SYSTEM_VARS__={'DEBUG':'off','TIMEOUT':30,'BACKSPACE':263,'TAB':9,'FASTUP':337,'FASTDOWN':336,'FLITERMODE':'half','DEBUGLEVEL':1,'__SAFEMODE__':'on','__STDOUT__':'$n0p','FLUSHCHECK':'off'}
+__COLORFILTERS__={'c3h7n3o9':[161,209,215,84,88,40,34,58],
 'rainbow':[161,209,215,192,84,88,82,40,28,56],
 'fak3':[483,484,485,486,487,488],
 'mengd@':[47,52,100,40,202],
@@ -62,84 +60,113 @@ __MAILSETTINGS__={
     'sender':'$n0p'
 }
 __PublicVarPool__={'LASTRET':'$n0p'}
-class LOG:
+class N0Out:
+    def write(self,string):
+        echo(string,_white)
+    def flush(self):
+        pass
+class Log:
     x=0
     y=0
-    hight=0
-    lenth=0
+    height=0
+    length=0
     total=0#总日志行数 [!]:非显示行数
     showEndLine=0#显示日志总行数
     limitCol=0
     limitLine=0
+    isVisible='on'
+    mode='left'
     color=_white
     showFlag=False
-    path=''
-    boarderX='-'
-    boarderY='|'
+    path='./'
+    borderX='-'
+    borderY='|'
     logName='Log'
     rawOutputLog=list()
     rawColor=list()
     showOutputLog=list()
     showColor=list()
-    def __init__(self,_x,_y,_lenth=20,_hight=7,_boarderX='-',_boarderY='|',_color=_white,_path='./'):
+    def __init__(self,_x,_y,_length=20,_height=7,_borderX='-',_borderY='|',_color=_white,_visible='on',_mode='left',_path='./'):
+        _x,_y,_length,_height=_legalSize(_x,_y,_length,_height)
         self.x=_x
         self.y=_y
-        self.hight=_hight
-        self.lenth=_lenth
-        self.limitCol=_lenth-2
-        self.limitLine=_hight-2
+        self.height=_height
+        self.length=_length
+        self.limitCol=_length-2
+        self.limitLine=_height-2
         self.path=_path
         self.showEndLine=0#初始化后showEndLine在0行
         self.color=_color
-        self.boarderX=_boarderX
-        self.boarderY=_boarderY
+        self.borderX=_borderX
+        self.borderY=_borderY
         self.rawOutputLog=list()#不在init里重新赋值的话两个LOG对象的变量会被python分配同一个地址
         self.rawColor=list()
         self.showOutputLog=list()
         self.showColor=list()
+        self.isVisible=_visible
+        self.mode=_mode
     def out(self,_string):
         self.rawOutputLog.append(str(_string))
-    def output(self,_string,colorStr='$n0p'): 
-        global __GRAPHIC__
+    def output(self,_string,colorStr='$n0p',show=True): 
+        escList=['\n']
         _string=str(_string)
-        if not __GRAPHIC__:
-            print(_string.replace('\n',' '))
-            return
+        self.rawOutputLog.append(_string)#原始日志数据
+        #_string=_string.replace('\n','')
+        _string=_string.replace('\r','')
+        _string=_string.replace('\t',' '*4)
         if _string=='':
             self.show()
-            return
-        #OUTPUT('[in out put]')
+            return '$n0p'
         if colorStr=='$n0p':
             colorStr=str(len(_string))+'|grey'
+        #OUTPUT('[in out put]')
         def delColor(__colorStr):
             rubbish,leftColorSt=divide(__colorStr,',')
             return leftColorSt
-        def addColor(__colorStr,__colorLenth,__colorWhat):
+        def addColor(__colorStr,__colorLength,__colorWhat):
             if __colorStr=='$n0p':
-                __colorStr=str(__colorLenth)+'|'+__colorWhat
+                __colorStr=str(__colorLength)+'|'+__colorWhat
             else:
-                __colorStr+=','+str(__colorLenth)+'|'+__colorWhat
+                __colorStr+=','+str(__colorLength)+'|'+__colorWhat
             return __colorStr
-        
-        self.rawOutputLog.append(str(_string))#原始日志数据
         self.rawColor.append(str(colorStr))#原始颜色信息
-        right=_string.replace('\n',' ')
         lines=0
         lineLengths=list()
+        right=_string
         while True:#切分过长字符串
-            if realLength(right)>self.limitCol:
-                left,right=_divideStrByRealLength(right,self.limitCol)
-                #OUTPUT(left+'<numberDividerReturn>'+right)缩到左侧=self.limitCol
+            escLocal=[]
+            for special in escList:
+                escLocal.append(findStr(right,special))
+            escLocal=remove(escLocal,'$n0p')
+            tem_escLocal=[]
+            for i in escLocal:#转为真实长度与边框比较
+                tem_escLocal.append(realLength(right[:i+1]))
+            escLocal=tem_escLocal
+            if len(escLocal)>0:
+                escLocal.append(self.limitCol)
+                preDivide=min(escLocal)
+                #if preDivide==self.limitCol:#如果边框长是最近换行，则应该交给无提前截取处理.因为limitCol是显示限制长度而不是第几个字符
+                #    preDivide='$n0p'
+            else:
+                preDivide='$n0p'
+            if preDivide!='$n0p':#进行提前截取
+                left,right=divideStrByRealLength(right,preDivide)
                 lineLengths.append(len(left))
                 self.showOutputLog.append(left)
-            else:
-                if right!='':
-                    self.showOutputLog.append(right)
-                    lineLengths.append(len(right))
-                    lines+=1
-                #OUTPUT('try out while')
-                break
-            #OUTPUT(left+'<outputWileTrue>'+right)
+            else:#无提前截取 按每行最长截取
+                if realLength(right)>self.limitCol:
+                    left,right=divideStrByRealLength(right,self.limitCol)
+                    #OUTPUT(left+'<numberDividerReturn>'+right)缩到左侧=self.limitCol
+                    lineLengths.append(len(left))
+                    self.showOutputLog.append(left)
+                else:
+                    if right!='':
+                        self.showOutputLog.append(right)
+                        lineLengths.append(len(right))
+                        lines+=1
+                    #OUTPUT('try out while')
+                    break
+                #OUTPUT(left+'<outputWileTrue>'+right)
             lines+=1
         self.showEndLine+=lines#向下移动显示结束行
         #OUTPUT('showEndLine:'+str(self.showEndLine))
@@ -155,24 +182,26 @@ class LOG:
                     #OUTPUT('leftColorStr into colors:'+str(colors))
                     leftColorStr='$n0p'#取出后置空
                     for color in colors:
-                        useColorLenth=0#本行当前颜色使用量
-                        colorLenth,colorWhat=divide(color,'|')
-                        colorLenth=beNumber(colorLenth)
-                        if colorLenth<=0:
-                                #leftColorStr=delColor(leftColorStr)#删掉一个颜色
-                                continue#如果当前颜色用尽则换下一个
+                        useColorLength=0#本行当前颜色使用量
+                        colorLength,colorWhat=divide(color,'|')
+                        colorLength=beNumber(colorLength)
+                        if colorLength=='$n0p':
+                            colorLength=0
+                        if colorLength<=0:
+                            #leftColorStr=delColor(leftColorStr)#删掉一个颜色
+                            continue#如果当前颜色用尽则换下一个
                         if lineLeft>0:
-                            if colorLenth>=lineLeft:#当前颜色超过本行
-                                useColorLenth=lineLeft
-                                colorLenth-=lineLeft#当前颜色剩余
-                                leftColorStr=addColor(leftColorStr,colorLenth,colorWhat)#没用完当写入leftColorStr
+                            if colorLength>=lineLeft:#当前颜色超过本行
+                                useColorLength=lineLeft
+                                colorLength-=lineLeft#当前颜色剩余
+                                leftColorStr=addColor(leftColorStr,colorLength,colorWhat)#没用完当写入leftColorStr
                                 lineLeft=0
                             else:#当前颜色不足本行
-                                useColorLenth=colorLenth
-                                lineLeft-=colorLenth
-                            newColorStr=addColor(newColorStr,useColorLenth,colorWhat)#使用了颜色
+                                useColorLength=colorLength
+                                lineLeft-=colorLength
+                            newColorStr=addColor(newColorStr,useColorLength,colorWhat)#使用了颜色
                         else:
-                            leftColorStr=addColor(leftColorStr,colorLenth,colorWhat)#未使用颜色直接写入leftColorStr
+                            leftColorStr=addColor(leftColorStr,colorLength,colorWhat)#未使用颜色直接写入leftColorStr
                         #OUTPUT('leftColorStr:'+str(leftColorStr))
                     #colors[index]=newColorStr#把填充后的颜色代码写回去
                 else:#无颜色部分默认灰色
@@ -184,54 +213,127 @@ class LOG:
         self.total+=1
         #OUTPUT('showOutputLog:'+str(self.showOutputLog))
         #OUTPUT('showColor:'+str(self.showColor))
-        self.show()
-    def clear(self):
-        for i in range(0,self.hight-2):
-            drawLine(self.x+1,self.y+i+1,' ',self.lenth-1,self.color)
-    def change(self,_x,_y,_lenth,_hight):
+        if show:
+            self.show()
+    def visible(self,state):
+        if state==True:
+            self.isVisible='on'
+        else:
+            self.clear(False)
+            self.isVisible='off'
+    def clear(self,onlyContent=True):
+        if self.isVisible!='off':
+            if onlyContent:
+                for i in range(0,self.height-2):
+                    drawLine(self.x+1,self.y+i+1,' ',self.length-1,self.color)
+            else:
+                for i in range(0,self.height-1):
+                    drawLine(self.x,self.y+i,' ',self.length,self.color)
+    def change(self,_x,_y,_length,_height):
+        self.clear(False)#完整清除
+        _x,_y,_length,_height=_legalSize(_x,_y,_length,_height)
         self.x=_x
         self.y=_y
-        self.hight=_hight
-        self.lenth=_lenth
-        self.limitCol=_lenth-2
-        self.limitLine=_hight-2
+        self.height=_height
+        self.length=_length
+        self.limitCol=_length-2
+        self.limitLine=_height-2
+        total=self.total
+        rawOutputLog=self.rawOutputLog
+        rawColor=self.rawColor
+        self.drop()
+        for i in range(0,total-2):#total计数的是下一个，所以减1，又因为从1数起，再减1
+            self.output(rawOutputLog[i],rawColor[i],False)
+        if self.isVisible!='off':
+            self.show()
+    def save(self):
+        outContent='### N0Shell Output Log ###\n'+nowTime()+'\n'+_version+'\n---[0]---\n'
+        index=1
+        for item in self.rawOutputLog:
+            outContent+=item+'\n---['+str(index)+']---\n'
+            index+=1
+        return createFile(self.path+self.logName,outContent)
     def draw(self,drawColor=-1):
-        if drawColor==-1:
-            drawColor=self.color
-        drawLine(self.x,self.y,self.boarderX,self.lenth,drawColor)
-        drawLine(self.x,self.y+self.hight-1,self.boarderX,self.lenth,drawColor)
-        verticalLine(self.x,self.y,self.boarderY,self.hight,drawColor)
-        verticalLine(self.x+self.lenth,self.y,self.boarderY,self.hight,drawColor)
+        if self.isVisible!='off':
+            if drawColor==-1:
+                drawColor=self.color
+            drawLine(self.x,self.y,self.borderX,self.length,drawColor)
+            drawLine(self.x,self.y+self.height-1,self.borderX,self.length,drawColor)
+            verticalLine(self.x,self.y,self.borderY,self.height,drawColor)
+            verticalLine(self.x+self.length,self.y,self.borderY,self.height,drawColor)
     def show(self,endLine=-1):
-        if not __GRAPHIC__:
-            return
-        def strColor2intColor(strColor):
+        if self.isVisible!='off':
+            def strColor2intColor(strColor):
+                    try:
+                        return eval('_'+strColor)
+                    except:
+                        try:
+                            return int(strColor)
+                        except:
+                            return _grey
             try:
-                return eval('_'+strColor)
-            except:
-                try:
-                    return int(strColor)
-                except:
-                    return _grey
-        if endLine==-1:
-            endLine=self.showEndLine
-        if endLine<self.limitLine:
-            startLine=0
-        else:
-            startLine=endLine-self.limitLine
-        #OUTPUT('startLine:'+str(startLine)+'|'+str(endLine))
-        self.clear()
-        outIndex=0
-        for lineId in range(startLine,endLine):
-            #OUTPUT('lineId:'+str(lineId))
-            showColors=listDivider(self.showColor[lineId],',')#每行输出前取出该行颜色指导字符串并切割
-            showPoint=0#重置输出指针
-            for color in showColors:
-                colorLenth,colorWhat=divide(color,'|')#切分出作用域和颜色
-                colorLenth=beNumber(colorLenth)
-                say(self.x+1+showPoint,self.y+1+outIndex,self.showOutputLog[lineId][showPoint:showPoint+colorLenth],strColor2intColor(colorWhat))#输出
-                showPoint+=colorLenth#输出指针后移
-            outIndex+=1
+                while True:
+                    if self.showFlag==True:
+                        time.sleep(0.01)
+                    else:
+                        break
+                self.showFlag=True
+                if endLine==-1:
+                    endLine=self.showEndLine
+                if endLine<self.limitLine:
+                    startLine=0
+                else:
+                    startLine=endLine-self.limitLine
+                #OUTPUT('startLine:'+str(startLine)+'|'+str(endLine))
+                #self.clear()
+                outLineIndex=0
+                for lineId in range(startLine,endLine):
+                    #OUTPUT('lineId:'+str(lineId))
+                    showPoint=0#重置输出指针(在字符串中的位置)
+                    showIndexX=0#在屏幕的位置
+                    colorStr=self.showColor[lineId]
+                    text=self.showOutputLog[lineId]
+                    if self.mode=='right':
+                        paddingLen=self.length-realLength(text)-1
+                        text=paddingLen*' '+text
+                        colorStr=str(paddingLen)+'|grey,'+colorStr
+                    elif self.mode=='middle':
+                        paddingLen=int((self.length-realLength(text)-1)/2)
+                        text=paddingLen*' '+text
+                        colorStr=str(paddingLen)+'|grey,'+colorStr
+                    elif self.mode=='mirror':
+                        paddingLen=self.length-realLength(text)-1
+                        rubbish=list(text)
+                        rubbish.reverse()
+                        text=paddingLen*' '+''.join(rubbish)
+                        showColors=colorStr.split(',')
+                        showColors.reverse()
+                        colorStr=','.join(showColors)
+                        colorStr=str(paddingLen)+'|grey,'+colorStr
+                    elif self.mode=='left':
+                        pass
+                    showColors=colorStr.split(',')#每行输出前取出该行颜色指导字符串并切割
+                    if __SYSTEM_VARS__['FLUSHCHECK']=='on':
+                        say(self.x+showIndexX+len(text),self.y+1+outLineIndex,'#'*(self.length-len(text)),self.color)#清理未输出
+                    else:
+                        say(self.x+showIndexX+len(text),self.y+1+outLineIndex,' '*(self.length-len(text)),_grey)#清理未输出
+                    for color in showColors:
+                        colorLength,colorWhat=divide(color,'|')#切分出作用域和颜色
+                        colorLength=beNumber(colorLength)#不用处理，发生异常会直接变灰
+                        say(self.x+1+showIndexX,self.y+1+outLineIndex,text[showPoint:showPoint+colorLength],strColor2intColor(colorWhat))#输出
+                        showIndexX+=realLength(text[showPoint:showPoint+colorLength])
+                        showPoint+=colorLength#输出指针后移
+                    outLineIndex+=1
+                self.showFlag=False
+            except Exception as e:
+                self.draw(_red)
+                if beNumber(systemVars('DEBUGLEVEL'),1)>=3:
+                    say(self.x,self.y,'===== OUTPUT ERROR ['+str(self.total)+']=== DETAIL '+str(e)+str(e.__traceback__.tb_lineno)+' === [PRESS KEY] =====',_red)
+                    _consoleWin.getch()
+                else:
+                    say(self.x,self.y,'===== OUTPUT ERROR ['+str(self.total)+']=== DETAIL '+str(e)+' =====',_red)
+                    time.sleep(1)
+                self.draw()
     def drop(self):
         self.total=0
         self.showEndLine=0
@@ -239,47 +341,55 @@ class LOG:
         self.rawColor=list()
         self.showOutputLog=list()
         self.showColor=list()
-    def setSize(self,_length=20,_hight=7):
-        self.hight=_hight
-        self.lenth=_length
-        self.limitCol=_length-2
-        self.limitLine=_hight-2
     def scroll(self):
-        nowEndLine=self.showEndLine
-        drawColor=229
-        drawLine(self.x,self.y,'-',self.lenth,drawColor)
-        drawLine(self.x,self.y+self.hight-1,'-',self.lenth,drawColor)
-        verticalLine(self.x,self.y,'}',self.hight,drawColor)
-        verticalLine(self.x+self.lenth,self.y,'{',self.hight,drawColor)
-        while True:
-            _press=consoleWin.getch()
-            if _press==ord('q'):
-                break
-            elif _press ==curses.KEY_UP:#上滚
-                nowEndLine-=1
-                if nowEndLine<self.limitLine:#请求显示的区域小于显示区域最后一行
-                    nowEndLine+=1
-                else:
-                    self.show(nowEndLine)
-            elif _press==curses.KEY_DOWN:#下滚
-                nowEndLine+=1
-                if nowEndLine>self.showEndLine:#请求显示的区域大于有记录的最后一行
+        if self.isVisible!='off':
+            nowEndLine=self.showEndLine
+            drawColor=229
+            drawLine(self.x,self.y,'-',self.length,drawColor)
+            drawLine(self.x,self.y+self.height-1,'-',self.length,drawColor)
+            verticalLine(self.x,self.y,'}',self.height,drawColor)
+            verticalLine(self.x+self.length,self.y,'{',self.height,drawColor)
+            while True:
+                _press=_consoleWin.getch()
+                if _press==ord('q'):
+                    break
+                elif _press==27:
+                    break
+                elif _press ==curses.KEY_UP:#上滚
                     nowEndLine-=1
-                else:
-                    self.show(nowEndLine)
-        self.draw()
-    def save(self):
-        global __N0Version__
-        if __N0Version__!='':
-            retVer=_version+'\\'+__N0Version__
-        else:
-            retVer=_version
-        outContent='### N0Lib '+retVer+' Output Log ###\n'+str(time.asctime( time.localtime(time.time())))+'\n---[0]---\n'
-        index=1
-        for item in self.rawOutputLog:
-            outContent+=item+'\n---['+str(index)+']---\n'
-            index+=1
-        createFile(self.path+self.logName,outContent)
+                    if nowEndLine<self.limitLine:#请求显示的区域小于显示区域最后一行
+                        nowEndLine+=1
+                    else:
+                        self.show(nowEndLine)
+                elif _press==__SYSTEM_VARS__['FASTUP']:#fast up
+                    if nowEndLine-int(self.height/2)<self.limitLine:#请求显示的区域小于显示区域最后一行
+                        nowEndLine-=1
+                        if nowEndLine<self.limitLine:
+                            nowEndLine+=1
+                        else:
+                            self.show(nowEndLine)
+                    else:
+                        nowEndLine-=int(self.height/2)
+                        self.show(nowEndLine)
+                elif _press==curses.KEY_DOWN:#下滚
+                    nowEndLine+=1
+                    if nowEndLine>self.showEndLine:#请求显示的区域大于有记录的最后一行
+                        nowEndLine-=1
+                    else:
+                        self.show(nowEndLine)
+                elif _press==__SYSTEM_VARS__['FASTDOWN']:#fast up
+                    if nowEndLine+int(self.height/2)>self.showEndLine:#请求显示的区域小于显示区域最后一行
+                        nowEndLine+=1
+                        if nowEndLine>self.showEndLine:
+                            nowEndLine-=1
+                        else:
+                            self.show(nowEndLine)
+                    else:
+                        nowEndLine+=int(self.height/2)
+                        self.show(nowEndLine)
+            self.draw()
+def nowTime():
+    return str(time.asctime( time.localtime(time.time())))
 def N0HELP(_function='$n0p'):
     WHO={
         'N0P3':'Developed by N0P3. Thank you. http://n0p3.cn',
@@ -306,7 +416,7 @@ def N0HELP(_function='$n0p'):
         'beNumber':'把字符串数字转为整型数字。(字符串数字)',
         'download':'下载文件。(链接,[保存文件名])',
         'publicVar':'查改增公开变量。(变量名,[值]) 当值为空时为获取变量的值。',
-        'divider':'按字符切割字符串。(字符串,切割字符) 把字符串按第一次出现的字符切成两半。',
+        'divide':'按字符切割字符串。(字符串,切割字符) 把字符串按第一次出现的字符切成两半。',
         'listDivider':'把字符串按字符切段。(字符串,切割字符) 返回list',
         'stringCompare':'忽略大小写判断两字符串是否相等。(字符串1,字符串2)',
         'splitPath':'切分路径。（路径) 依次返回文件名和剩余路径',
@@ -326,8 +436,8 @@ def N0HELP(_function='$n0p'):
         'zipDir':'把一个文件夹压缩成zip。(文件夹路径,压缩后文件名)',
         'unzip':'解压zip。(zip文件路径,解压路径)',
         'nargv':'从命令行获取参数。',
-        'setFliter':'设置滤镜。(滤镜名,[滤镜模式]) 模式默认为half，不会对输出者造成影响，改为all将对所有consoleOut结果造成影响。',
-        'addFliter':'添加新的滤镜。(滤镜名,颜色列表)',
+        'setFilter':'设置滤镜。(滤镜名,[滤镜模式]) 模式默认为half，不会对输出者造成影响，改为all将对所有consoleOut结果造成影响。',
+        'addFilter':'添加新的滤镜。(滤镜名,颜色列表)',
         'newArgv':'将命令行参数向前偏移。(开始保留的参数序号) 某些情况下会用到。',
     }
     if _function !='$n0p':
@@ -338,7 +448,7 @@ def N0HELP(_function='$n0p'):
         else:
             echo("No command '"+_function -+"'.",_red)
     else:
-        _viewDict(OutputWin,HELPS)
+        _viewDict(__SYSTEM_VARS__['__STDOUT__'],HELPS)
         consoleOut('LOG','日志类非常有用，所有的输出都可以交给它。',_blue,_white)
         consoleOut('LOG','change(x,y,length,hight)修改窗口左上角坐标，长度和高度','TAB',_yellow)
         consoleOut('LOG','draw(color)画出边框，参数是颜色','TAB',_yellow)
@@ -374,6 +484,44 @@ def _divideStrByRealLength(string,targetLength):
 def verticalLine(_x,_y,TheChar,length,fColor=_grey):
     for i in range(0,length):
         say(_x,_y+i,TheChar,fColor)
+def _legalSize(x,y,length,height):
+    _x,_y,_length,_height=legalSize(x,y,length,height)
+    if _x=='$n0p':
+        safeExit('SIZE','Calculate size error.x:'+str(x)+' y:'+str(y)+' length:'+str(length)+' height:'+str(height))
+    return _x,_y,_length,_height
+def remove(_list,what):
+    newList=[]
+    for i in _list:
+        if i !=what:
+            newList.append(i)
+    return newList
+def legalSize(x,y,length,height):
+    #如果出现错误将返回四个$n0p
+    nowColumns,nowLines=terminalSize()
+    if x>=nowColumns:
+        x=nowColumns-length
+    if y>=nowLines:
+        y=nowLines-height
+    if x<=0:
+        x=1
+    if y<=0:
+        y=1
+    if x+length>=nowColumns:
+        length=nowColumns-x-1
+    if y+height>=nowLines:
+        height=nowLines-y
+    if length<=0 or height<=0:
+        return '$n0p','$n0p','$n0p','$n0p'
+    return x,y,length,height
+def terminalSize():
+    terminal=os.get_terminal_size()
+    columns=terminal.columns
+    lines=terminal.lines
+    return columns,lines
+def terminalClean(cleanChar=' ',color=grey):
+    length,height=terminalSize()
+    for i in range(0,height):
+        say(1,i,cleanChar*length,color)
 def _viewDict(_window,_dict=dict()):
     for _key in _dict:
         _window.output(_key+':'+_dict[_key],str(len(_key)+1)+'|119,'+str(len(_dict[_key]))+'|white')
@@ -401,23 +549,23 @@ def say(x,y,word,fColor=_white,clear=False,clearChar=' '):
             y=y-1
             try: 
                 fColor=int(fColor)
-                consoleWin.addstr(y,x,word,_setColor(fColor))
+                _consoleWin.addstr(y,x,word,_setColor(fColor))
                 __OUTX__=x+1+len(word)
                 __OUTY__=y+1
             except: 
                 consoleOut('Old out','SAY excuted error.')
         else:
             consoleOut('Old out','[SAY]: Error posite.<'+str(x)+','+str(y)+'>')
-        consoleWin.refresh()
+        _consoleWin.refresh()
     else:
         if __DEBUG__:
             print('[SAY]:ERROR! Please use initGraphic().')
 def _setColor(fontColor,backColor=-1):
     return curses.color_pair(fontColor)
-def flowSay(word,fColor=_white):
+def followSay(word,fColor=_white):
     say(__OUTX__,__OUTY__,word,fColor)
-def echo(content,color=_white,**kv):
-    consoleOut('[Egg_002]:How can you see me!',content,'SPACE',color,COLORFLITER=True)
+def echo(content,color=_white,STDOUT='$n0p'):
+    consoleOut('[Egg_002]:How can you see me!',content,'SPACE',color,COLORFILTER=True,STDOUT=STDOUT)
 def beNumber(ToBeNumber):
     try:
         ToBeNumber=int(ToBeNumber)
@@ -438,6 +586,7 @@ def sizeFormat(size, isDisk=False, precision=2):
             return f'{round(size, precision)}{i}'
     return f'{round(size, precision)}{i}'
 def download(url,fileName='$n0p'):
+    OutputWin=__SYSTEM_VARS__['__STDOUT__']
     size=0
     chunk_size=1024
     if fileName=='$n0p':
@@ -499,6 +648,17 @@ def divide(string,TheChar,delESC=False):
         return left,'$n0p'
     else:
         return left,right
+def divideStrByRealLength(string,targetLength):
+    real_length=0
+    retStr=''
+    index=0
+    for i in string:
+        index+=1
+        real_length+=getWidth(i)
+        retStr+=i
+        if real_length>=targetLength:
+            return retStr,string[index:]
+    return '>>RETURN<<LINEHEAD>>FAILED<<','>>RETURN<<LINEHEAD>>FAILED<<'
 def listDivider(strTargetList,TheChar):
     retList=list()
     leftOne,rightOne=divide(strTargetList,TheChar)
@@ -657,26 +817,6 @@ def tagExtractor(tagName,targetStr,tagParamStr='',unknowParam=False):
             return rawStr[dataStart+1:dataEnd]
         except Exception as e:
             return '$n0p'
-def legalPath(_path):
-    if _path!='':
-        if _path[-1]!=__dirChar__:
-            _path+=__dirChar__
-        _path=_path.replace('N0:/',__WORKPATH__)#N0目录转linux目录
-        if len(_path)>2:
-            _path=_path[:2]+_path[2:].replace('./','')#去当前目录
-    return _path
-def realPath(_path,isFile=False):
-    retPath=legalPath(_path)
-    if retPath=='$n0p/':#当前shell目录
-        targetPath=__SHELLPATH__
-    elif retPath[0]=='/':#绝对路径
-        targetPath=retPath
-    else:#相对路径
-        targetPath=__SHELLPATH__+retPath
-    if isFile:
-        return targetPath[:-1]
-    else:
-        return targetPath
 def randomString(_length):
     randomStr = ''
     baseStr = 'ABCDEFGHIGKLMNOPQRSTUVWXYZabcdefghigklmnopqrstuvwxyz0123456789'
@@ -802,16 +942,32 @@ def flagExtractor(string,_flagFormat):
             debugOut("flagExtractor","Flags search finished.")
             break
     return flags
+def systemVars(varName,value='$n0p'):
+    value=beNumber(value,value)
+    if value=='$n0p':
+        try:
+            if varName[0]=='_':#下划线开头系统变量不可读
+                debugOut('systemVars',varName+" can't be read.",_red,_red)
+                return '$n0p'
+            return __SYSTEM_VARS__[varName]
+        except:
+            debugOut('systemVars',"Not found '"+varName+"'.",_red,_white)
+            return '$n0p'
+    else:
+        try:
+            __SYSTEM_VARS__[varName]=value
+        except:
+            consoleOut('systemVars',"Not found '"+varName+"'.",_red,_white)
 def patchDict(_oldDict,_patch):
     #更新dict中已有的，新增没有的.无法删除.
     for item in _patch:
         _oldDict[item]=_patch[item]
     return _oldDict
-def rainbowOut(identity,content,leftColor=_grey,colorList=[161,209,215,84,88,40,34,58],colorStart=-1,colorEnd=-1,output=True,COLORFLITER=True):
+def rainbowOut(identity,content,leftColor=_grey,colorList=[161,209,215,84,88,40,34,58],colorStart=-1,colorEnd=-1,output=True,COLORFILTER=True,STDOUT='$n0p'):
     retColorStr='0|grey'
     colorIndex=0
     try:
-        colorList=__COLORFLITERS__[colorList]
+        colorList=__COLORFILTERS__[colorList]
     except:
         pass
     if colorStart>=0 and colorEnd>=0:
@@ -826,14 +982,14 @@ def rainbowOut(identity,content,leftColor=_grey,colorList=[161,209,215,84,88,40,
         for i in range(colorStart,colorEnd):
             colorList.append(i)
     SKIP=False
-    lenth=len(content)
+    length=len(content)
     Loop=True
     while Loop:
         if colorList==[]:
-            retColorStr=str(lenth)+'|grey'
+            retColorStr=str(length)+'|grey'
             Loop=False
         elif len(colorList)==1:
-            retColorStr=str(lenth)+'|'+str(colorList[0])
+            retColorStr=str(length)+'|'+str(colorList[0])
             Loop=False
         else:
             for color in colorList:
@@ -841,7 +997,7 @@ def rainbowOut(identity,content,leftColor=_grey,colorList=[161,209,215,84,88,40,
                     SKIP=False
                     continue
                 else:
-                    if colorIndex<=lenth:
+                    if colorIndex<=length:
                         retColorStr+=',1|'+str(color)
                         colorIndex+=1
                     else:
@@ -850,16 +1006,26 @@ def rainbowOut(identity,content,leftColor=_grey,colorList=[161,209,215,84,88,40,
         colorList.reverse()
         SKIP=True
     if output:
-        consoleOut(identity,content,leftColor,colorStr=retColorStr,COLORFLITER=COLORFLITER)
+        consoleOut(identity,content,leftColor,colorStr=retColorStr,COLORFILTER=COLORFILTER)
     return retColorStr
-def debugOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n0p'):
-    if __DEBUG__:
-        consoleOut(identity,_content,leftColor,rightColor,colorStr,False)#debug信息不受影响
-def consoleOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n0p',COLORFLITER=True):
+def debugOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n0p',LEVEL=1,STDOUT='$n0p'):
+    LEVEL=beNumber(LEVEL,1)
+    if systemVars('DEBUG')=='on':
+        if beNumber(systemVars('DEBUGLEVEL'),1)>=LEVEL:
+            consoleOut(identity,_content,leftColor,rightColor,colorStr,False,STDOUT)#debug信息不受影响
+def consoleOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n0p',COLORFILTER=True,STDOUT='$n0p'):
+    #2021.3.31 SPACE在COLORFILTER开模式下不正常，TAB缩进不正常
     identity=str(identity)
+    identity=identity.replace('\t',' '*4)
     _content=str(_content)
+    _content=_content.replace('\t',' '*4)
     leftColor=str(leftColor)
     rightColor=str(rightColor)
+    #SAY(1,1,_content,_yellow)
+    #consoleWin.getch()
+    #SAY(1,1,'OK',_green)
+    if STDOUT=='$n0p':
+        STDOUT=__SYSTEM_VARS__['__STDOUT__']
     TAB=False
     SPACE=False
     if leftColor=='TAB':
@@ -869,7 +1035,7 @@ def consoleOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n
         leftColor='grey'
         SPACE=True
     def identityColor(identity):
-        if not SPACE and __FLITERMODE__!='all' or not COLORFLITER:
+        if not SPACE and (__SYSTEM_VARS__['FILTERMODE']!='all' or not COLORFILTER):
             if identity=='WARNING':
                 colorStr='10|yellow,'
             elif identity=='ERROR':
@@ -881,17 +1047,17 @@ def consoleOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n
             elif identity=='+' or identity=='C':
                 colorStr='4|white,'
             else:
-                colorStr=str(len(identity)+4)+'|'+leftColor+','
+                colorStr=str(len(identity)+3)+'|'+leftColor+','
         else:
             colorStr=''
         return colorStr
-    if __COLORFLITER__!='$n0p' and COLORFLITER:
+    if __COLORFILTER__!='$n0p' and COLORFILTER:
         try:
-            if __FLITERMODE__=='all':
+            if __SYSTEM_VARS__['FILTERMODE']=='all':
                 #如果滤镜模式是all的话，rainbowOut生成的颜色还需要算上len(identity)+4个字符
-                colorStr=rainbowOut(identity,identity+'N0P3'+_content,leftColor,__COLORFLITERS__[__COLORFLITER__],output=False)
+                colorStr=rainbowOut(identity,identity+'N0P3'+_content,leftColor,__COLORFILTERS__[__COLORFILTER__],output=False,STDOUT=STDOUT)
             else:
-                colorStr=rainbowOut(identity,_content,leftColor,__COLORFLITERS__[__COLORFLITER__],output=False)
+                colorStr=rainbowOut(identity,_content,leftColor,__COLORFILTERS__[__COLORFILTER__],output=False,STDOUT=STDOUT)
         except:
             pass
     if colorStr=='$n0p':#不启用颜色指导字符串
@@ -903,11 +1069,11 @@ def consoleOut(identity,_content,leftColor='grey',rightColor='grey',colorStr='$n
         colorStr=identityColor(identity)
         colorStr+=_keepColorStr
     if SPACE:
-        OutputWin.output(_content,colorStr)
+        STDOUT.output(_content,colorStr)
     elif TAB:
-        OutputWin.output((len(identity)+4)*' '+_content,colorStr)
+        STDOUT.output((len(identity)+3)*' '+_content,colorStr)
     else:
-        OutputWin.output("["+identity+"]: "+_content,colorStr)
+        STDOUT.output("["+identity+"]:"+_content,colorStr)
 def N0DEBUG(_bool):
     global __DEBUG__
     if _bool==True:
@@ -961,13 +1127,24 @@ def nargv(id):
     except:
         ret='$n0p'
     return ret
-def setFliter(name,mode='half'):
-    global __COLORFLITER__,__FLITERMODE__
-    __COLORFLITER__=name
-    __FLITERMODE__=mode
-def addFliter(name,colorList):
-    global __COLORFLITERS__
-    __COLORFLITERS__[name]=colorList
+def findStr(string,subStr,begin=0,end='$n0p'):
+    if end=='$n0p':
+        end=len(string)
+    try:
+        return string.index(subStr,begin,end)
+    except:
+        return '$n0p'
+def setFilter(name,mode='half'):
+    global __COLORFILTER__,__FILTERMODE__
+    __COLORFILTER__=name
+    __FILTERMODE__=mode
+def addFilter(name,colorList):
+    global __COLORFILTERS__
+    __COLORFILTERS__[name]=colorList
+def safeExit(word=""):
+    endGraphic()
+    print(word)
+    sys.exit(0)
 def newArgv(offset):
     index=offset
     retParts=list()
@@ -978,10 +1155,11 @@ def newArgv(offset):
         else:
             return retParts
         index+=1
-def initGraphic():
-    global consoleWin,__limitX__,__limitY__,__GRAPHIC__
+def initGraphic(window="$n0p"):
+    global _consoleWin,__limitX__,__limitY__,__GRAPHIC__
     if not __GRAPHIC__:
-        consoleWin = curses.initscr()
+        n0out=N0Out()
+        _consoleWin = curses.initscr()
         curses.start_color()
         curses.use_default_colors()
         for i in range(0, curses.COLORS):
@@ -991,21 +1169,23 @@ def initGraphic():
         __limitX__=curses.COLS-1
         curses.noecho()
         curses.cbreak()
-        consoleWin.keypad(True)
+        _consoleWin.keypad(True)
+        sys.stdout=n0out
+        if window=="$n0p":
+            __SYSTEM_VARS__['__STDOUT__']=Log(1,1,*terminalSize)
+        else:
+            if type(window)==Log:
+                __SYSTEM_VARS__['__STDOUT__']=window
+            else:
+                safeExit("InitGraphic need a Log object.")
         __GRAPHIC__=True
-    else:
-        pass
-def safeExit():
-    endGraphic()
-    sys.exit(0)
+
 def endGraphic():
     global __GRAPHIC__
     if __GRAPHIC__:
         __GRAPHIC__=False
         curses.nocbreak()
-        consoleWin.keypad(False)
+        _consoleWin.keypad(False)
         curses.echo()
         curses.endwin()
     cursor.show()
-initGraphic()
-OutputWin=LOG(1,1,_path='./N0Log/')
\ No newline at end of file
